//	CurvedSpacesMouse.c
//
//	© 2009 by Jeff Weeks
//	See TermsOfUse.txt

#include "CurvedSpaces-Common.h"


void MouseMove(
	ModelData	*md,
	double		dx,
	double		dy,
	bool		aShiftKeyIsDown,
	bool		aCtrlKeyIsDown,
	bool		anAltKeyIsDown)
{
	Matrix	theIncrement;

	//	The user typically uses the mouse to navigate.
	//	As an exceptional case, if the alt key is down
	//	mouse motion instead serves to move the centerpiece.
	if (anAltKeyIsDown)
	{
#ifdef CENTERPIECE_DISPLACEMENT
		//	Interpret the motion relative to the centerpiece's
		//	own local coordinate system.
		if (aCtrlKeyIsDown)
			MatrixTranslation(&theIncrement, md->itsSpaceType,  0,  0, dy);
		else
			MatrixTranslation(&theIncrement, md->itsSpaceType, dx, dy,  0);

		//	Pre-multiply by theIncrement.
		MatrixProduct(&theIncrement, &md->itsCenterpiecePlacement, &md->itsCenterpiecePlacement);
		
		//	Stay in the fundamental domain.
		if (md->itsDirichletDomain != NULL)
			StayInDirichletDomain(md->itsDirichletDomain, &md->itsCenterpiecePlacement);

		//	Keep numerical errors from accumulating, so we stay
		//	in Isom(S³) = O(4), Isom(E³) or Isom(H³) = O(3,1).
		FastGramSchmidt(&md->itsCenterpiecePlacement, md->itsSpaceType);
#endif
	}
	else
	{
		//	Allow full six-degrees-of-freedom navigation.
		if (aShiftKeyIsDown)
		{
			//	Translate
			if (aCtrlKeyIsDown)
				MatrixTranslation(&theIncrement, md->itsSpaceType,  0,  0, dy);
			else
				MatrixTranslation(&theIncrement, md->itsSpaceType, dx, dy,  0);
		}
		else
		{
			//	Rotate
			if (aCtrlKeyIsDown)
				MatrixRotation(&theIncrement,   0,  0, -dx);
			else
				MatrixRotation(&theIncrement, -dy, dx,   0);
		}

		//	Pre-multiply by theIncrement.
		MatrixProduct(&theIncrement, &md->itsUserPlacement, &md->itsUserPlacement);

		//	Keep numerical errors from accumulating, so we stay
		//	in Isom(S³) = O(4), Isom(E³) or Isom(H³) = O(3,1).
		FastGramSchmidt(&md->itsUserPlacement, md->itsSpaceType);
	}
	
	//	Ask the idle-time routine to redraw the scene.
	md->itsRedrawRequestFlag = true;
}
