//	CurvedSpacesUtilities.c
//
//	© 2009 by Jeff Weeks
//	See TermsOfUse.txt

#include "CurvedSpaces-Common.h"


ErrorText GetAbsolutePath(
	const char		*aDirectory,	//	input,  zero-terminated UTF-8 string, may be NULL
	const char		*aFileName,		//	input,  zero-terminated UTF-8 string, may be NULL
	char			*aPath,			//	output, zero-terminated UTF-8 string
	unsigned int	aPathSize)		//	output buffer size, including space for terminating zero
{
	//	Assemble an absolute path of the form
	//
	//		<base path>/<directory name>/<file name>
	//
	//	where
	//
	//		<base path> says where the application lives,
	//			and won't be known until runtime,
	//
	//		<directory name> specifies the directory of interest,
	//			for example "Languages", "Sounds" or "Textures".
	//
	//		<file name> specifies the particular file,
	//			for example "StarterCode-ja.txt" or "Clouds.rgba".

	char	*theTerminator,
			theChar;

	//	Get the base path.
	if (GetBasePath(aPath, aPathSize) == false)
	{
		aPath[0] = 0;
		return "Couldn't get base path in GetAbsolutePath()";
	}
	
	//	Locate the terminating zero (which GetBasePath() guarantees).
	theTerminator = aPath;
	while (*theTerminator != 0)
		theTerminator++;

	//	Append aDirectory.
	if (aDirectory != NULL)
	{
		while ((theChar = *aDirectory++) != 0)
		{
			*theTerminator = theChar;
			if (++theTerminator < aPath + aPathSize)
				*theTerminator = 0;
			else
			{
				aPath[0] = 0;
				return "Path name too long (1) in  in GetAbsolutePath()";
			}
		}
		
		//	Append a path separator.
		*theTerminator = '/';
		if (++theTerminator < aPath + aPathSize)
			*theTerminator = 0;
		else
		{
			aPath[0] = 0;
			return "Path name too long (2) in  in GetAbsolutePath()";
		}
	}

	//	Append aFileName.
	if (aFileName != NULL)
	{
		while ((theChar = *aFileName++) != 0)
		{
			*theTerminator = theChar;
			if (++theTerminator < aPath + aPathSize)
				*theTerminator = 0;
			else
			{
				aPath[0] = 0;
				return "Path name too long (3) in  in GetAbsolutePath()";
			}
		}
	}

	return NULL;
}
