//	CurvedSpacesMacOSView.h
//
//	© 2009 by Jeff Weeks
//	See TermsOfUse.txt

#import <Cocoa/Cocoa.h>
#include "CurvedSpaces-Common.h"

//	A quick-and-dirty hack for saving the frames.
//#define SAVE_ANIMATION


@interface CurvedSpacesMacOSView : NSOpenGLView
{
	IBOutlet NSPanel		*itsPreferencesPanel;
	IBOutlet NSTabView		*itsPrefTabView;

	IBOutlet NSBox			*itsPrefGeneralBox;
	IBOutlet NSTextField	*itsPrefWindowWidthLabel,
							*itsPrefWindowWidth;
	IBOutlet NSTextField	*itsPrefWindowDistanceLabel,
							*itsPrefWindowDistance;
	IBOutlet NSBox			*itsPrefStereo3DBox;
	IBOutlet NSButton		*itsPrefStereo3D;
	IBOutlet NSTextField	*itsPrefEyeSeparationLabel,
							*itsPrefEyeSeparation,
							*itsPrefEyeSeparationUnits;
	IBOutlet NSTextField	*itsPrefIntrinsicUnitLabel,
							*itsPrefIntrinsicUnit,
							*itsPrefIntrinsicUnitUnits;

	IBOutlet NSButton		*itsPrefFog,
							*itsPrefFrameRate,
							*itsPrefSyncToMonitor,
							*itsPrefMultisampling,
							*itsPrefAnisotropic,
							*itsPrefShaders;

	IBOutlet NSButton		*itsPrefOK;	

	//	An NSTimer supports continuous redraw.
	NSTimer					*itsTimer;
	
	//	The Microseconds clock lets us keep the animation smooth
	//	regardless of the frame rate.
	long long				itsLastRenderTime;
	
	//	If we're displaying the frame rate in the title bar,
	//	we'll want to update it only about once per second.
	long long				itsLastFrameRateTime;
	
	//	Have we commandeered the mouse for steering the spaceship?
	bool					itsMouseSteeringFlag;
	
	//	Cocoa automatically remembers the "last directory" but really
	//	we need two "last directories", one for the last matrix file opened
	//	and one for the last TIFF image saved.
	NSMutableString			*itsLastMatrixDirectory,
							*itsLastImageDirectory;

	//	The ModelData contains all the information about the space itself
	//	along with all platform-independent information about how
	//	it is to be displayed.
	ModelData				md;
}
+ (NSOpenGLPixelFormat*)pixelFormatSupportingMultisampling:(bool)aMultisampleFlag;
- (id)initWithFrame:(NSRect)frameRect;
- (void)dealloc;
- (void)awakeFromNib;
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;
- (BOOL)acceptsFirstResponder;
- (void)setRedrawWithSync:(bool)aSyncFlag;
- (void)pleaseRedraw;
- (void)keyDown:(NSEvent *)anEvent;
- (void)insertText:(id)aString;
- (BOOL)performKeyEquivalent:(NSEvent *)anEvent;
- (void)mouseDown:(NSEvent *)anEvent;
- (void)rightMouseUp:(NSEvent *)anEvent;
- (void)enterNavigationalMode;
- (void)exitNavigationalMode;
- (void)mouseMoved:(NSEvent *)anEvent;
- (void)scrollWheel:(NSEvent *)anEvent;
- (void)drawRect:(NSRect)bounds;
- (BOOL)validateMenuItem:(NSMenuItem *)aMenuItem;

- (IBAction)commandFileOpen:(id)sender;
- (IBAction)commandFileExportCopy:(id)sender;
- (IBAction)commandFileExportSave:(id)sender;

- (IBAction)commandViewCenterpieceNone:(id)sender;
- (IBAction)commandViewCenterpieceEarth:(id)sender;
- (IBAction)commandViewCenterpieceGalaxy:(id)sender;
- (IBAction)commandViewCenterpieceGyroscope:(id)sender;
- (IBAction)commandViewObserver:(id)sender;
- (IBAction)commandViewColorCoding:(id)sender;
- (IBAction)commandViewCliffordParallels:(id)sender;
- (IBAction)commandViewVertexFigures:(id)sender;
- (IBAction)commandViewFog:(id)sender;
- (IBAction)commandViewMultisampling:(id)sender;
- (IBAction)commandViewGraphicsFixedFunction:(id)sender;
- (IBAction)commandViewGraphicsShaders:(id)sender;
- (IBAction)commandViewStereoscopic3D:(id)sender;

- (IBAction)commandLanguageDe:(id)sender;
- (IBAction)commandLanguageEn:(id)sender;
- (IBAction)commandLanguageFr:(id)sender;
- (IBAction)commandLanguageJa:(id)sender;

- (IBAction)commandHelpHelp:(id)sender;

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename;
- (IBAction)setUpPreferencesPanel:(id)sender;
- (void)setStereoLabels;
- (IBAction)readOnePreference:(id)sender;
- (void)replaceOpenGLContext;
- (IBAction)shutDownPreferencesPanel:(id)sender;
- (NSImage *)contentsAsImage;
- (NSData *)contentsAsTIFF;
- (NSBitmapImageRep *)contentsAsBitmapImageRep;
- (void)invertRawImage: (unsigned int *)aPixelBuffer pixelsWide:(unsigned int)aWidth pixelsHigh:(unsigned int)aHeight;
- (void)changeLanguage:(NSString *)aLanguageCode;
- (NSMenu *)buildLocalizedMenuBar;
- (NSMenu *)addToMenu:(NSMenu *)aParentMenu submenuWithTitle:(char *)aSubmenuTitle;
- (void)setPreferencePanelLabels;
- (void)reshape;
- (void)windowWillMiniaturize:(NSNotification *)aNotification;
- (BOOL)windowShouldClose:(id)sender;
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication;
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender;
- (void)applicationWillTerminate:(NSNotification *)aNotification;

#ifdef SAVE_ANIMATION
- (IBAction)qtStartRecording:(id)sender;
- (IBAction)qtStopRecording:(id)sender;
- (IBAction)qtPreviewMovie:(id)sender;
- (IBAction)qtRenderMovie:(id)sender;
- (IBAction)qtSize720x480:(id)sender;
- (IBAction)qtSize1920x1080:(id)sender;
- (IBAction)qtSize960x540:(id)sender;
#endif

@end
