//	CurvedSpaces-Win32.h
//
//	Definitions for Curved Spaces' Win32 user interface.
//	The user interface knows about the ModelData,
//	but doesn't know anything about OpenGL.
//
//	© 2009 by Jeff Weeks
//	See TermsOfUse.txt

#pragma once

//	Include platform-independent definitions (excluding OpenGL).
#include "CurvedSpaces-Common.h"

//	Include the Win32 headers.
//	Request the WIN32_LEAN_AND_MEAN headers
//	using 16-bit UNICODE characters.
#define WIN32_LEAN_AND_MEAN
#define UNICODE
#include <windows.h>


//	The user never sees the window class names.
#define MAIN_WINDOW_CLASS_NAME		L"Curved Spaces Main Window Class"
#define DRAWING_WINDOW_CLASS_NAME	L"Curved Spaces Drawing Window Class"


//	All the information needed to render a scene
//	lives in a single struct attached to the window.
typedef struct
{
	//	The main window.
	HWND		itsWindow;
	
	//	Draw into a child window of the main window.
	HWND		itsDrawingPanel;

	//	Basic OpenGL rendering context.
	HDC			itsDeviceContext;
	HGLRC		itsRenderingContext;
	
	//	Keep track of the file name (without the path or extension)
	//	so we can use it as a window title.
	WCHAR		itsFileTitle[64];
	
	//	Keep track of the active tab in the preferences dialog, if any.
	HWND		itsPreferencesPane;
	
	//	Keep all platform-independent data in a ModelData struct
	//	(see CurvedSpaces-Common.h for details).
	ModelData	md;

} WindowData;


//	Win32-specific global variables

//	An "atom" uniquely identifies the main window class.
extern ATOM			gMainWindowClass;

//	The user's preferred directory for saving images may differ
//	from his/her preferred directory for reading manifolds.
extern WCHAR		gLastImageDirectory[MAX_PATH],
					gLastManifoldDirectory[MAX_PATH];


//	Win32-specific global functions

//	in CurvedSpaces-Win32-WinMain.c
extern HWND				CreateCurvedSpacesWindow(void);
extern BOOL CALLBACK	SetWindowTitle(HWND aWindow, LPARAM aTitle);

//	in CurvedSpaces-Win32-WndProc.c
extern LRESULT CALLBACK	MainWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
extern LRESULT CALLBACK	DrawingWndProc(HWND aDrawingWindow, UINT uMsg, WPARAM wParam, LPARAM lParam);
extern void				DoFileOpen(WindowData *wd);
extern BOOL CALLBACK	RefreshLanguage(HWND aWindow, LPARAM anUnusedParameter);

//	in CurvedSpaces-Win32-Utilities.c
extern WCHAR			*GetLocalizedTextAsWCHAR(const char *aKey, WCHAR *aValueBuffer, unsigned int aValueBufferLength);
extern bool				LanguageReadsRightToLeft(void);

//	in AVIMovie.c (optional)
extern ErrorText		MakeOpenGLMovie(WindowData *wd);
