//	GeometryGames-Win32-LoadEntryPoints.c
//
//	WindowsXP/Vista/7 still uses the same OpenGL 1.1 library that it used back in 1996 (!).
//	To access OpenGL 1.2 and later on Win32 we must use the "extension" mechanism.
//	The following code loads the needed entry points.
//
//	Our program's Win32 GUI calls LoadOpenGLFunctions() to load the entry points,
//	but never sees those entry points.   Indeed, the platform-dependent GUIs
//	don't see OpenGL at all.  Only our platform-independent code sees OpenGL
//	and makes calls using the function pointers loaded below along with
//	a few of the built-in OpenGL 1.1 calls (but very few of the latter
//	while using programmable shaders).
//
//	© 2009 by Jeff Weeks
//	See TermsOfUse.txt

#define WIN32_LEAN_AND_MEAN
#define UNICODE
#include <windows.h>
#include "GeometryGames-Win32-LoadEntryPoints.h"
#include "GeometryGames-Win32-OpenGLEntryPoints.h"


unsigned int LoadOpenGLFunctions(void)
{
	//	Try to load each function without regard for whether
	//	the extension is supported or not.
	//	Elsewhere we'll check which extensions are available.
	//	If an extension isn't supported we should get NULL 
	//	for the function address, but we don't rely on this.

	glDrawRangeElements			= (PFNGLDRAWRANGEELEMENTSPROC)			wglGetProcAddress("glDrawRangeElements"			);
	glGenBuffers				= (PFNGLGENBUFFERSPROC)					wglGetProcAddress("glGenBuffers"				);
	glBindBuffer				= (PFNGLBINDBUFFERPROC)					wglGetProcAddress("glBindBuffer"				);
	glDeleteBuffers				= (PFNGLDELETEBUFFERSPROC)				wglGetProcAddress("glDeleteBuffers"				);
	glBufferData				= (PFNGLBUFFERDATAPROC)					wglGetProcAddress("glBufferData"				);
	glCreateShader				= (PFNGLCREATESHADERPROC)				wglGetProcAddress("glCreateShader"				);
	glShaderSource				= (PFNGLSHADERSOURCEPROC)				wglGetProcAddress("glShaderSource"				);
	glCompileShader				= (PFNGLCOMPILESHADERPROC)				wglGetProcAddress("glCompileShader"				);
	glGetShaderiv				= (PFNGLGETSHADERIVPROC)				wglGetProcAddress("glGetShaderiv"				);
	glGetShaderInfoLog			= (PFNGLGETSHADERINFOLOGPROC)			wglGetProcAddress("glGetShaderInfoLog"			);
	glCreateProgram				= (PFNGLCREATEPROGRAMPROC)				wglGetProcAddress("glCreateProgram"				);
	glAttachShader				= (PFNGLATTACHSHADERPROC)				wglGetProcAddress("glAttachShader"				);
	glBindAttribLocation		= (PFNGLBINDATTRIBLOCATIONPROC)			wglGetProcAddress("glBindAttribLocation"		);
	glLinkProgram				= (PFNGLLINKPROGRAMPROC)				wglGetProcAddress("glLinkProgram"				);
	glGetProgramiv				= (PFNGLGETPROGRAMIVPROC)				wglGetProcAddress("glGetProgramiv"				);
	glGetProgramInfoLog			= (PFNGLGETPROGRAMINFOLOGPROC)			wglGetProcAddress("glGetProgramInfoLog"			);
	glUseProgram				= (PFNGLUSEPROGRAMPROC)					wglGetProcAddress("glUseProgram"				);
	glDeleteShader				= (PFNGLDELETESHADERPROC)				wglGetProcAddress("glDeleteShader"				);
	glDeleteProgram				= (PFNGLDELETEPROGRAMPROC)				wglGetProcAddress("glDeleteProgram"				);
	glGetUniformLocation		= (PFNGLGETUNIFORMLOCATIONPROC)			wglGetProcAddress("glGetUniformLocation"		);
	glUniform1i					= (PFNGLUNIFORM1IPROC)					wglGetProcAddress("glUniform1i"					);
	glUniform1f					= (PFNGLUNIFORM1FPROC)					wglGetProcAddress("glUniform1f"					);
	glUniform2f					= (PFNGLUNIFORM2FPROC)					wglGetProcAddress("glUniform2f"					);
	glUniform3f					= (PFNGLUNIFORM3FPROC)					wglGetProcAddress("glUniform3f"					);
	glUniform4f					= (PFNGLUNIFORM4FPROC)					wglGetProcAddress("glUniform4f"					);
	glUniform1fv				= (PFNGLUNIFORM1FVPROC)					wglGetProcAddress("glUniform1fv"				);
	glUniform2fv				= (PFNGLUNIFORM2FVPROC)					wglGetProcAddress("glUniform2fv"				);
	glUniform3fv				= (PFNGLUNIFORM3FVPROC)					wglGetProcAddress("glUniform3fv"				);
	glUniform4fv				= (PFNGLUNIFORM4FVPROC)					wglGetProcAddress("glUniform4fv"				);
	glUniformMatrix4fv			= (PFNGLUNIFORMMATRIX4FVPROC)			wglGetProcAddress("glUniformMatrix4fv"			);
	glVertexAttribPointer		= (PFNGLVERTEXATTRIBPOINTERPROC)		wglGetProcAddress("glVertexAttribPointer"		);
	glEnableVertexAttribArray	= (PFNGLENABLEVERTEXATTRIBARRAYPROC)	wglGetProcAddress("glEnableVertexAttribArray"	);
	glDisableVertexAttribArray	= (PFNGLDISABLEVERTEXATTRIBARRAYPROC)	wglGetProcAddress("glDisableVertexAttribArray"	);
	glVertexAttrib1f			= (PFNGLVERTEXATTRIB1FPROC)				wglGetProcAddress("glVertexAttrib1f"			);
	glVertexAttrib2f			= (PFNGLVERTEXATTRIB2FPROC)				wglGetProcAddress("glVertexAttrib2f"			);
	glVertexAttrib3f			= (PFNGLVERTEXATTRIB3FPROC)				wglGetProcAddress("glVertexAttrib3f"			);
	glVertexAttrib4f			= (PFNGLVERTEXATTRIB4FPROC)				wglGetProcAddress("glVertexAttrib4f"			);
	glVertexAttrib1fv			= (PFNGLVERTEXATTRIB1FVPROC)			wglGetProcAddress("glVertexAttrib1fv"			);
	glVertexAttrib2fv			= (PFNGLVERTEXATTRIB2FVPROC)			wglGetProcAddress("glVertexAttrib2fv"			);
	glVertexAttrib3fv			= (PFNGLVERTEXATTRIB3FVPROC)			wglGetProcAddress("glVertexAttrib3fv"			);
	glVertexAttrib4fv			= (PFNGLVERTEXATTRIB4FVPROC)			wglGetProcAddress("glVertexAttrib4fv"			);

	//	If we didn't find the functions under their permanent names,
	//	try to load them using their former extension names.
	if (glDrawRangeElements	== NULL)
		glDrawRangeElements			= (PFNGLDRAWRANGEELEMENTSPROC)			wglGetProcAddress("glDrawRangeElementsEXT"			);
	if (glGenBuffers == NULL)
		glGenBuffers				= (PFNGLGENBUFFERSPROC)					wglGetProcAddress("glGenBuffersARB"					);
	if (glBindBuffer == NULL)
		glBindBuffer				= (PFNGLBINDBUFFERPROC)					wglGetProcAddress("glBindBufferARB"					);
	if (glDeleteBuffers == NULL)
		glDeleteBuffers				= (PFNGLDELETEBUFFERSPROC)				wglGetProcAddress("glDeleteBuffersARB"				);
	if (glBufferData == NULL)
		glBufferData				= (PFNGLBUFFERDATAPROC)					wglGetProcAddress("glBufferDataARB"					);
	if (glCreateShader == NULL)
		glCreateShader				= (PFNGLCREATESHADERPROC)				wglGetProcAddress("glCreateShaderObjectARB"			);
	if (glShaderSource == NULL)
		glShaderSource				= (PFNGLSHADERSOURCEPROC)				wglGetProcAddress("glShaderSourceARB"				);
	if (glCompileShader == NULL)
		glCompileShader				= (PFNGLCOMPILESHADERPROC)				wglGetProcAddress("glCompileShaderARB"				);
	if (glGetShaderiv == NULL)
		glGetShaderiv				= (PFNGLGETSHADERIVPROC)				wglGetProcAddress("glGetObjectParameterivARB"		);
	if (glGetShaderInfoLog == NULL)
		glGetShaderInfoLog			= (PFNGLGETSHADERINFOLOGPROC)			wglGetProcAddress("glGetInfoLogARB"					);
	if (glCreateProgram == NULL)
		glCreateProgram				= (PFNGLCREATEPROGRAMPROC)				wglGetProcAddress("glCreateProgramObjectARB"		);
	if (glAttachShader == NULL)
		glAttachShader				= (PFNGLATTACHSHADERPROC)				wglGetProcAddress("glAttachObjectARB"				);
	if (glBindAttribLocation == NULL)
		glBindAttribLocation		= (PFNGLBINDATTRIBLOCATIONPROC)			wglGetProcAddress("glBindAttribLocationARB"			);
	if (glLinkProgram == NULL)
		glLinkProgram				= (PFNGLLINKPROGRAMPROC)				wglGetProcAddress("glLinkProgramARB"				);
	if (glGetProgramiv == NULL)
		glGetProgramiv				= (PFNGLGETPROGRAMIVPROC)				wglGetProcAddress("glGetProgramivARB"				);
	if (glGetProgramInfoLog == NULL)
		glGetProgramInfoLog			= (PFNGLGETPROGRAMINFOLOGPROC)			wglGetProcAddress("glGetInfoLogARB"					);
	if (glUseProgram == NULL)
		glUseProgram				= (PFNGLUSEPROGRAMPROC)					wglGetProcAddress("glUseProgramObjectARB"			);
	if (glDeleteShader == NULL)
		glDeleteShader				= (PFNGLDELETESHADERPROC)				wglGetProcAddress("glDeleteObjectARB"				);
	if (glDeleteProgram == NULL)
		glDeleteProgram				= (PFNGLDELETEPROGRAMPROC)				wglGetProcAddress("glDeleteObjectARB"				);
	if (glGetUniformLocation == NULL)
		glGetUniformLocation		= (PFNGLGETUNIFORMLOCATIONPROC)			wglGetProcAddress("glGetUniformLocationARB"			);
	if (glUniform1i == NULL)
		glUniform1i					= (PFNGLUNIFORM1IPROC)					wglGetProcAddress("glUniform1iARB"					);
	if (glUniform1f == NULL)
		glUniform1f					= (PFNGLUNIFORM1FPROC)					wglGetProcAddress("glUniform1fARB"					);
	if (glUniform2f == NULL)
		glUniform2f					= (PFNGLUNIFORM2FPROC)					wglGetProcAddress("glUniform2fARB"					);
	if (glUniform3f == NULL)
		glUniform3f					= (PFNGLUNIFORM3FPROC)					wglGetProcAddress("glUniform3fARB"					);
	if (glUniform4f == NULL)
		glUniform4f					= (PFNGLUNIFORM4FPROC)					wglGetProcAddress("glUniform4fARB"					);
	if (glUniform1fv == NULL)
		glUniform1fv				= (PFNGLUNIFORM1FVPROC)					wglGetProcAddress("glUniform1fvARB"					);
	if (glUniform2fv == NULL)
		glUniform2fv				= (PFNGLUNIFORM2FVPROC)					wglGetProcAddress("glUniform2fvARB"					);
	if (glUniform3fv == NULL)
		glUniform3fv				= (PFNGLUNIFORM3FVPROC)					wglGetProcAddress("glUniform3fvARB"					);
	if (glUniform4fv == NULL)
		glUniform4fv				= (PFNGLUNIFORM4FVPROC)					wglGetProcAddress("glUniform4fvARB"					);
	if (glUniformMatrix4fv == NULL)
		glUniformMatrix4fv			= (PFNGLUNIFORMMATRIX4FVPROC)			wglGetProcAddress("glUniformMatrix4fvARB"			);
	if (glVertexAttribPointer == NULL)
		glVertexAttribPointer		= (PFNGLVERTEXATTRIBPOINTERPROC)		wglGetProcAddress("glVertexAttribPointerARB"		);
	if (glEnableVertexAttribArray == NULL)
		glEnableVertexAttribArray	= (PFNGLENABLEVERTEXATTRIBARRAYPROC)	wglGetProcAddress("glEnableVertexAttribArrayARB"	);
	if (glDisableVertexAttribArray == NULL)
		glDisableVertexAttribArray	= (PFNGLDISABLEVERTEXATTRIBARRAYPROC)	wglGetProcAddress("glDisableVertexAttribArrayARB"	);
	if (glVertexAttrib1f == NULL)
		glVertexAttrib1f			= (PFNGLVERTEXATTRIB1FPROC)				wglGetProcAddress("glVertexAttrib1fARB"				);
	if (glVertexAttrib2f == NULL)
		glVertexAttrib2f			= (PFNGLVERTEXATTRIB2FPROC)				wglGetProcAddress("glVertexAttrib2fARB"				);
	if (glVertexAttrib3f == NULL)
		glVertexAttrib3f			= (PFNGLVERTEXATTRIB3FPROC)				wglGetProcAddress("glVertexAttrib3fARB"				);
	if (glVertexAttrib4f == NULL)
		glVertexAttrib4f			= (PFNGLVERTEXATTRIB4FPROC)				wglGetProcAddress("glVertexAttrib4fARB"				);
	if (glVertexAttrib1fv == NULL)
		glVertexAttrib1fv			= (PFNGLVERTEXATTRIB1FVPROC)			wglGetProcAddress("glVertexAttrib1fvARB"			);
	if (glVertexAttrib2fv == NULL)
		glVertexAttrib2fv			= (PFNGLVERTEXATTRIB2FVPROC)			wglGetProcAddress("glVertexAttrib2fvARB"			);
	if (glVertexAttrib3fv == NULL)
		glVertexAttrib3fv			= (PFNGLVERTEXATTRIB3FVPROC)			wglGetProcAddress("glVertexAttrib3fvARB"			);
	if (glVertexAttrib4fv == NULL)
		glVertexAttrib4fv			= (PFNGLVERTEXATTRIB4FVPROC)			wglGetProcAddress("glVertexAttrib4fvARB"			);


	//	If we couldn't load the Vertex Buffer Object functions
	//	under either their permanent names or their extension names,
	//	we're out of luck.  The caller must tell the user what happened,
	//	and then quit.
	return (glDrawRangeElements	!= NULL	//	not VBO related, but we want to assume it's there
		 && glGenBuffers		!= NULL
		 && glBindBuffer		!= NULL
		 && glDeleteBuffers		!= NULL
		 && glBufferData		!= NULL);
}
