//	GeometryGames-OpenGLEntryPoints.c
//
//	WindowsXP/Vista/7 still uses the same OpenGL 1.1 library that it used back in 1996 (!).
//	To access OpenGL 1.2 and later on Win32 we must use the "extension" mechanism.
//	The following code defines the needed entry points.
//
//	Technical Note:  The same platform-independent code compiles a little differently
//	under Win32 than under Mac OS X.  Under Mac OS X, the program compiles against
//	the Open GL 2.0 (or later) headers, and a call to, say, glVertexAttribPointer()
//	gets resolved by automatic (soft) linking to the built-in OpenGL library's 
//	implementation of glVertexAttribPointer().  Under WinXP, the token
//	glVertexAttribPointer is treated not like an ordinary function that gets
//	linked at program launch, but rather as a program-defined global variable,
//	which happens to be a function pointer.  Ultimately the effect is the same.
//
//	© 2009 by Jeff Weeks
//	See TermsOfUse.txt

#include "GeometryGames-Win32-OpenGLEntryPoints.h"


static void __stdcall	DummyDeleteBuffers(signed int n, const unsigned int *buffers);
static void __stdcall	DummyUseProgram(unsigned int aProgram);
static void __stdcall	DummyDeleteProgram(unsigned int aProgram);

PFNGLDRAWRANGEELEMENTSPROC			glDrawRangeElements			= NULL;
PFNGLGENBUFFERSPROC					glGenBuffers				= NULL;
PFNGLBINDBUFFERPROC					glBindBuffer				= NULL;
PFNGLDELETEBUFFERSPROC				glDeleteBuffers				= &DummyDeleteBuffers;
PFNGLBUFFERDATAPROC					glBufferData				= NULL;
PFNGLCREATESHADERPROC				glCreateShader				= NULL;
PFNGLSHADERSOURCEPROC				glShaderSource				= NULL;
PFNGLCOMPILESHADERPROC				glCompileShader				= NULL;
PFNGLGETSHADERIVPROC				glGetShaderiv				= NULL;
PFNGLGETSHADERINFOLOGPROC			glGetShaderInfoLog			= NULL;
PFNGLCREATEPROGRAMPROC				glCreateProgram				= NULL;
PFNGLATTACHSHADERPROC				glAttachShader				= NULL;
PFNGLBINDATTRIBLOCATIONPROC			glBindAttribLocation		= NULL;
PFNGLLINKPROGRAMPROC				glLinkProgram				= NULL;
PFNGLGETPROGRAMIVPROC				glGetProgramiv				= NULL;
PFNGLGETPROGRAMINFOLOGPROC			glGetProgramInfoLog			= NULL;
PFNGLUSEPROGRAMPROC					glUseProgram				= &DummyUseProgram;
PFNGLDELETESHADERPROC				glDeleteShader				= NULL;
PFNGLDELETEPROGRAMPROC				glDeleteProgram				= &DummyDeleteProgram;
PFNGLGETUNIFORMLOCATIONPROC			glGetUniformLocation		= NULL;
PFNGLUNIFORM1IPROC					glUniform1i					= NULL;
PFNGLUNIFORM1FPROC					glUniform1f					= NULL;
PFNGLUNIFORM2FPROC					glUniform2f					= NULL;
PFNGLUNIFORM3FPROC					glUniform3f					= NULL;
PFNGLUNIFORM4FPROC					glUniform4f					= NULL;
PFNGLUNIFORM1FVPROC					glUniform1fv				= NULL;
PFNGLUNIFORM2FVPROC					glUniform2fv				= NULL;
PFNGLUNIFORM3FVPROC					glUniform3fv				= NULL;
PFNGLUNIFORM4FVPROC					glUniform4fv				= NULL;
PFNGLUNIFORMMATRIX4FVPROC			glUniformMatrix4fv			= NULL;
PFNGLVERTEXATTRIBPOINTERPROC		glVertexAttribPointer		= NULL;
PFNGLENABLEVERTEXATTRIBARRAYPROC	glEnableVertexAttribArray	= NULL;
PFNGLDISABLEVERTEXATTRIBARRAYPROC	glDisableVertexAttribArray	= NULL;
PFNGLVERTEXATTRIB1FPROC				glVertexAttrib1f			= NULL;
PFNGLVERTEXATTRIB2FPROC				glVertexAttrib2f			= NULL;
PFNGLVERTEXATTRIB3FPROC				glVertexAttrib3f			= NULL;
PFNGLVERTEXATTRIB4FPROC				glVertexAttrib4f			= NULL;
PFNGLVERTEXATTRIB1FVPROC			glVertexAttrib1fv			= NULL;
PFNGLVERTEXATTRIB2FVPROC			glVertexAttrib2fv			= NULL;
PFNGLVERTEXATTRIB3FVPROC			glVertexAttrib3fv			= NULL;
PFNGLVERTEXATTRIB4FVPROC			glVertexAttrib4fv			= NULL;


//	If an error occurs at startup, the program will call its OpenGL shutdown code.
//	On the Mac it's harmless to call a function like glDeleteBuffers() when
//	no OpenGL context is active, but on Win32 glDeleteBuffers might still be NULL.
//	Rather than dirtying up the platform-independent code with a test 
//	"if glDeleteBuffers != NULL", I'd prefer to dirty up the Win32 code
//	by initializing glDeleteBuffers to &DummyDeleteBuffers rather than to NULL, 
//	so we can always call it safely.  If Windows ever lets us compile 
//	for OpenGL 1.5 or later, the function DummyDeleteBuffers() can disappear
//	along with the code for loading the glDeleteBuffers entry point at runtime.
//	Similar comments apply to the vertex shader functions dummified below.

static void __stdcall DummyDeleteBuffers(
	signed int			n,
	const unsigned int	*buffers)
{
	(void) n;
	(void) buffers;
}

static void __stdcall DummyUseProgram(
	unsigned int	aProgram)
{
	(void) aProgram;
}

static void __stdcall DummyDeleteProgram(
	unsigned int	aProgram)
{
	(void) aProgram;
}
